/* eslint-disable @typescript-eslint/naming-convention */ "use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "DeclarativeChart", {
    enumerable: true,
    get: function() {
        return DeclarativeChart;
    }
});
const _interop_require_wildcard = require("@swc/helpers/_/_interop_require_wildcard");
const _react = /*#__PURE__*/ _interop_require_wildcard._(require("react"));
const _chartutilities = require("@fluentui/chart-utilities");
const _reacttheme = require("@fluentui/react-theme");
const _reactsharedcontexts = require("@fluentui/react-shared-contexts");
const _tokens = require("@fluentui/tokens");
const _d3color = /*#__PURE__*/ _interop_require_wildcard._(require("d3-color"));
const _PlotlySchemaAdapter = require("./PlotlySchemaAdapter");
const _index = require("../DonutChart/index");
const _index1 = require("../VerticalStackedBarChart/index");
const _index2 = require("../LineChart/index");
const _index3 = require("../HorizontalBarChartWithAxis/index");
const _index4 = require("../AreaChart/index");
const _index5 = require("../HeatMapChart/index");
const _SankeyChart = require("../SankeyChart/SankeyChart");
const _index6 = require("../GaugeChart/index");
const _index7 = require("../GroupedVerticalBarChart/index");
const _index8 = require("../VerticalBarChart/index");
const _imageExporter = require("./imageExporter");
const _index9 = require("../ScatterChart/index");
const _withResponsiveContainer = require("../ResponsiveContainer/withResponsiveContainer");
const ResponsiveDonutChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index.DonutChart);
const ResponsiveVerticalStackedBarChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index1.VerticalStackedBarChart);
const ResponsiveLineChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index2.LineChart);
const ResponsiveHorizontalBarChartWithAxis = (0, _withResponsiveContainer.withResponsiveContainer)(_index3.HorizontalBarChartWithAxis);
const ResponsiveAreaChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index4.AreaChart);
const ResponsiveHeatMapChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index5.HeatMapChart);
const ResponsiveSankeyChart = (0, _withResponsiveContainer.withResponsiveContainer)(_SankeyChart.SankeyChart);
const ResponsiveGaugeChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index6.GaugeChart);
const ResponsiveGroupedVerticalBarChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index7.GroupedVerticalBarChart);
const ResponsiveVerticalBarChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index8.VerticalBarChart);
const ResponsiveScatterChart = (0, _withResponsiveContainer.withResponsiveContainer)(_index9.ScatterChart);
const useColorMapping = ()=>{
    const colorMap = _react.useRef(new Map());
    return colorMap;
};
const useIsDarkTheme = ()=>{
    const parentV9Theme = _react.useContext(_reactsharedcontexts.ThemeContext_unstable);
    const v9Theme = parentV9Theme ? parentV9Theme : _tokens.webLightTheme;
    // Get background and foreground colors
    const backgroundColor = _d3color.hsl(v9Theme.colorNeutralBackground1);
    const foregroundColor = _d3color.hsl(v9Theme.colorNeutralForeground1);
    const isDarkTheme = backgroundColor.l < foregroundColor.l;
    return isDarkTheme;
};
const DeclarativeChart = /*#__PURE__*/ _react.forwardRef((props, forwardedRef)=>{
    const { plotlySchema } = (0, _chartutilities.sanitizeJson)(props.chartSchema);
    const chart = (0, _chartutilities.mapFluentChart)(plotlySchema);
    if (!chart.isValid) {
        throw new Error(`Invalid chart schema: ${chart.errorMessage}`);
    }
    let plotlyInput = plotlySchema;
    try {
        plotlyInput = (0, _chartutilities.decodeBase64Fields)(plotlyInput);
    } catch (error) {
        throw new Error(`Failed to decode plotly schema: ${error}`);
    }
    const plotlyInputWithValidData = {
        ...plotlyInput,
        data: chart.validTracesInfo.map((trace)=>plotlyInput.data[trace.index])
    };
    let { selectedLegends } = plotlySchema;
    const colorMap = useColorMapping();
    const isDarkTheme = useIsDarkTheme();
    const chartRef = _react.useRef(null);
    if (!(0, _chartutilities.isArrayOrTypedArray)(selectedLegends)) {
        selectedLegends = [];
    }
    const [activeLegends, setActiveLegends] = _react.useState(selectedLegends);
    const onActiveLegendsChange = (keys)=>{
        setActiveLegends(keys);
        if (props.onSchemaChange) {
            props.onSchemaChange({
                plotlySchema: {
                    plotlyInput,
                    selectedLegends: keys
                }
            });
        }
    };
    _react.useEffect(()=>{
        // eslint-disable-next-line @typescript-eslint/no-shadow
        const { plotlySchema } = (0, _chartutilities.sanitizeJson)(props.chartSchema);
        // eslint-disable-next-line @typescript-eslint/no-shadow
        const { selectedLegends } = plotlySchema;
        setActiveLegends(selectedLegends !== null && selectedLegends !== void 0 ? selectedLegends : []);
    }, [
        props.chartSchema
    ]);
    const multiSelectLegendProps = {
        canSelectMultipleLegends: true,
        onChange: onActiveLegendsChange,
        selectedLegends: activeLegends
    };
    const commonProps = {
        legendProps: multiSelectLegendProps,
        componentRef: chartRef
    };
    const renderLineAreaScatter = (plotlyData, isAreaChart)=>{
        var _plotlyData_;
        const isScatterMarkers = ((_plotlyData_ = plotlyData[0]) === null || _plotlyData_ === void 0 ? void 0 : _plotlyData_.mode) === 'markers';
        const chartProps = {
            ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToScatterChartProps)({
                data: plotlyData,
                layout: plotlyInput.layout
            }, isAreaChart, colorMap, isDarkTheme),
            ...commonProps
        };
        if (isAreaChart) {
            return /*#__PURE__*/ _react.createElement(ResponsiveAreaChart, chartProps);
        }
        if (isScatterMarkers) {
            return /*#__PURE__*/ _react.createElement(ResponsiveScatterChart, chartProps);
        }
        return /*#__PURE__*/ _react.createElement(ResponsiveLineChart, chartProps);
    };
    const checkAndRenderChart = (isAreaChart = false)=>{
        let fallbackVSBC = false;
        const xValues = plotlyInputWithValidData.data[0].x;
        const isXDate = (0, _chartutilities.isDateArray)(xValues);
        const isXNumber = (0, _chartutilities.isNumberArray)(xValues);
        const isXMonth = (0, _chartutilities.isMonthArray)(xValues);
        // Consider year as categorical variable not numeric continuous variable
        // Also year is not considered a date variable as it is represented as a point
        // in time and brings additional complexity of handling timezone and locale
        // formatting given the current design of the charting library
        const isXYear = (0, _chartutilities.isYearArray)(xValues);
        if ((isXDate || isXNumber) && !isXYear) {
            return renderLineAreaScatter(plotlyInputWithValidData.data, isAreaChart);
        } else if (isXMonth) {
            const updatedData = plotlyInputWithValidData.data.map((dataPoint)=>({
                    ...dataPoint,
                    x: (0, _PlotlySchemaAdapter.correctYearMonth)(dataPoint.x)
                }));
            return renderLineAreaScatter(updatedData, isAreaChart);
        }
        // Unsupported schema, render as VerticalStackedBarChart
        fallbackVSBC = true;
        return /*#__PURE__*/ _react.createElement(ResponsiveVerticalStackedBarChart, {
            ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToVSBCProps)(plotlyInputWithValidData, colorMap, isDarkTheme, fallbackVSBC),
            ...commonProps
        });
    };
    // TODO
    const exportAsImage = _react.useCallback((opts)=>{
        var _chartRef_current;
        return (0, _imageExporter.toImage)((_chartRef_current = chartRef.current) === null || _chartRef_current === void 0 ? void 0 : _chartRef_current.chartContainer, {
            background: _reacttheme.tokens.colorNeutralBackground1,
            scale: 5,
            ...opts
        });
    }, []);
    _react.useImperativeHandle(props.componentRef, ()=>({
            exportAsImage
        }), [
        exportAsImage
    ]);
    switch(chart.type){
        case 'donut':
            return /*#__PURE__*/ _react.createElement(ResponsiveDonutChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToDonutProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'horizontalbar':
            return /*#__PURE__*/ _react.createElement(ResponsiveHorizontalBarChartWithAxis, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToHorizontalBarWithAxisProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'groupedverticalbar':
            return /*#__PURE__*/ _react.createElement(ResponsiveGroupedVerticalBarChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToGVBCProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'verticalstackedbar':
            return /*#__PURE__*/ _react.createElement(ResponsiveVerticalStackedBarChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToVSBCProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'heatmap':
            return /*#__PURE__*/ _react.createElement(ResponsiveHeatMapChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToHeatmapProps)(plotlyInputWithValidData),
                ...commonProps,
                legendProps: {}
            });
        case 'sankey':
            return /*#__PURE__*/ _react.createElement(ResponsiveSankeyChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToSankeyProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'gauge':
            return /*#__PURE__*/ _react.createElement(ResponsiveGaugeChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToGaugeProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        case 'verticalbar':
            return /*#__PURE__*/ _react.createElement(ResponsiveVerticalBarChart, {
                ...(0, _PlotlySchemaAdapter.transformPlotlyJsonToVBCProps)(plotlyInputWithValidData, colorMap, isDarkTheme),
                ...commonProps
            });
        // TODO: Add 'scatter' as a separate chart type
        case 'area':
        case 'line':
        case 'fallback':
            // Need recheck for area chart as we don't have ability to check for valid months in previous step
            const isAreaChart = plotlyInputWithValidData.data.some((series)=>series.fill === 'tonexty' || series.fill === 'tozeroy' || !!series.stackgroup);
            return checkAndRenderChart(isAreaChart);
        default:
            var _plotlyInputWithValidData_data_;
            throw new Error(`Unsupported chart type :${(_plotlyInputWithValidData_data_ = plotlyInputWithValidData.data[0]) === null || _plotlyInputWithValidData_data_ === void 0 ? void 0 : _plotlyInputWithValidData_data_.type}`);
    }
});
DeclarativeChart.displayName = 'DeclarativeChart';
