import * as React from 'react';
import { useGroupedVerticalBarChartStyles_unstable } from './useGroupedVerticalBarChartStyles.styles';
import { select as d3Select } from 'd3-selection';
import { max as d3Max, min as d3Min } from 'd3-array';
import { scaleBand as d3ScaleBand } from 'd3-scale';
import { useId } from '@fluentui/react-utilities';
import { ChartTypes, getAccessibleDataObject, tooltipOfAxislabels, XAxisTypes, getTypeOfAxis, formatScientificLimitWidth, getScalePadding, getBarWidth, isScalePaddingDefined, getNextColor, areArraysEqual, calculateLongestLabelWidth, useRtl } from '../../utilities/index';
import { CartesianChart, Legends, DataVizPalette, getColorFromToken } from '../../index';
const MIN_DOMAIN_MARGIN = 8;
const X1_INNER_PADDING = 0.1;
// x1_inner_padding = space_between_bars / (space_between_bars + bar_width)
// => space_between_bars = (x1_inner_padding / (1 - x1_inner_padding)) * bar_width
/** Rate at which the space between the bars in a group changes wrt the bar width */ const BAR_GAP_RATE = X1_INNER_PADDING / (1 - X1_INNER_PADDING);
const VERTICAL_BAR_GAP = 1;
const MIN_BAR_HEIGHT = 1;
export const GroupedVerticalBarChart = /*#__PURE__*/ React.forwardRef((props = {
    maxBarWidth: 24
}, forwardedRef)=>{
    var _props_legendProps, _props_legendProps1;
    const _tooltipId = useId('GVBCTooltipId_');
    const _emptyChartId = useId('_GVBC_empty');
    const _useRtl = useRtl();
    let _domainMargin = MIN_DOMAIN_MARGIN;
    let _keys = [];
    let _xAxisLabels = [];
    let _datasetForBars = [];
    let _margins = {
        top: 0,
        right: 0,
        bottom: 0,
        left: 0
    };
    let _groupedVerticalBarGraph = [];
    let _refArray = [];
    let _yMax = 0;
    let _calloutAnchorPoint = null;
    let _barWidth = 0;
    let _groupWidth = 0;
    let _xAxisInnerPadding = 0;
    let _xAxisOuterPadding = 0;
    const cartesianChartRef = React.useRef(null);
    const Y_ORIGIN = 0;
    const [color, setColor] = React.useState('');
    const [dataForHoverCard, setDataForHoverCard] = React.useState(0);
    const [selectedLegends, setSelectedLegends] = React.useState(((_props_legendProps = props.legendProps) === null || _props_legendProps === void 0 ? void 0 : _props_legendProps.selectedLegends) || []);
    const [xCalloutValue, setXCalloutValue] = React.useState('');
    const [yCalloutValue, setYCalloutValue] = React.useState('');
    const [YValueHover, setYValueHover] = React.useState([]);
    const [hoverXValue, setHoverXValue] = React.useState('');
    const [calloutLegend, setCalloutLegend] = React.useState('');
    const [activeLegend, setActiveLegend] = React.useState('');
    const [callOutAccessibilityData, setCallOutAccessibilityData] = React.useState(undefined);
    const [clickPosition, setClickPosition] = React.useState({
        x: 0,
        y: 0
    });
    const [isPopoverOpen, setPopoverOpen] = React.useState(false);
    const classes = useGroupedVerticalBarChartStyles_unstable(props);
    React.useEffect(()=>{
        var _props_legendProps;
        if (!areArraysEqual((_props_legendProps = props.legendProps) === null || _props_legendProps === void 0 ? void 0 : _props_legendProps.selectedLegends, selectedLegends)) {
            var _props_legendProps1;
            setSelectedLegends(((_props_legendProps1 = props.legendProps) === null || _props_legendProps1 === void 0 ? void 0 : _props_legendProps1.selectedLegends) || []);
        }
    }, [
        (_props_legendProps1 = props.legendProps) === null || _props_legendProps1 === void 0 ? void 0 : _props_legendProps1.selectedLegends
    ]);
    React.useImperativeHandle(props.componentRef, ()=>{
        var _cartesianChartRef_current;
        var _cartesianChartRef_current_chartContainer;
        return {
            chartContainer: (_cartesianChartRef_current_chartContainer = (_cartesianChartRef_current = cartesianChartRef.current) === null || _cartesianChartRef_current === void 0 ? void 0 : _cartesianChartRef_current.chartContainer) !== null && _cartesianChartRef_current_chartContainer !== void 0 ? _cartesianChartRef_current_chartContainer : null
        };
    }, []);
    const _adjustProps = ()=>{
        _barWidth = getBarWidth(props.barWidth, props.maxBarWidth);
        // x0_inner_padding = space_between_groups / (space_between_groups + group_width)
        // space_between_groups = 2 * bar_width
        // group_width = keys.length * bar_width + (keys.length - 1) * space_between_bars
        _xAxisInnerPadding = getScalePadding(props.xAxisInnerPadding, undefined, 2 / (2 + keys.length + (keys.length - 1) * BAR_GAP_RATE));
        _xAxisOuterPadding = getScalePadding(props.xAxisOuterPadding);
    };
    const _createDataset = (points)=>{
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const datasetForBars = [];
        const dataset = [];
        points.forEach((point, index)=>{
            const singleDatasetPoint = {};
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const singleDatasetPointForBars = {};
            const singleDataSeries = [];
            point.series.forEach((seriesPoint, seriesIndex)=>{
                singleDatasetPoint[seriesPoint.key] = seriesPoint.data;
                singleDatasetPointForBars[seriesPoint.key] = {
                    ...seriesPoint
                };
                singleDataSeries.push(seriesPoint);
            });
            singleDatasetPointForBars.xAxisPoint = point.name;
            singleDatasetPointForBars.indexNum = index;
            singleDatasetPointForBars.groupSeries = singleDataSeries;
            singleDatasetPointForBars.stackCallOutAccessibilityData = point.stackCallOutAccessibilityData;
            datasetForBars.push(singleDatasetPointForBars);
            dataset.push(singleDatasetPoint);
        });
        return datasetForBars;
    };
    const _createDataSetOfGVBC = (points)=>{
        const keys = [];
        const xAxisLabels = points.map((singlePoint)=>singlePoint.name);
        points[0].series.forEach((singleKey)=>{
            keys.push(singleKey.key);
        });
        const datasetForBars = _createDataset(points);
        return {
            keys,
            xAxisLabels,
            datasetForBars
        };
    };
    const onLegendSelectionChange = (selectedLegends, event, currentLegend)=>{
        var _props_legendProps, _props_legendProps1;
        if ((_props_legendProps = props.legendProps) === null || _props_legendProps === void 0 ? void 0 : _props_legendProps.canSelectMultipleLegends) {
            setSelectedLegends(selectedLegends);
        } else {
            setSelectedLegends(selectedLegends.slice(-1));
        }
        if ((_props_legendProps1 = props.legendProps) === null || _props_legendProps1 === void 0 ? void 0 : _props_legendProps1.onChange) {
            props.legendProps.onChange(selectedLegends, event, currentLegend);
        }
    };
    const _getLegendData = (points)=>{
        const data = points;
        const defaultPalette = [
            getColorFromToken(DataVizPalette.color1),
            getColorFromToken(DataVizPalette.color2),
            getColorFromToken(DataVizPalette.color3),
            getColorFromToken(DataVizPalette.color4),
            getColorFromToken(DataVizPalette.color5)
        ];
        const actions = [];
        data.forEach((singleChartData)=>{
            singleChartData.series.forEach((point)=>{
                let color = point.color ? point.color : defaultPalette[Math.floor(Math.random() * 4 + 1)];
                const checkSimilarLegends = actions.filter((leg)=>leg.title === point.legend && leg.color === color);
                if (checkSimilarLegends.length > 0) {
                    return;
                }
                const legend = {
                    title: point.legend,
                    color,
                    hoverAction: ()=>{
                        _handleChartMouseLeave();
                        _onLegendHover(point.legend);
                    },
                    onMouseOutAction: ()=>{
                        _onLegendLeave();
                    }
                };
                actions.push(legend);
            });
        });
        return /*#__PURE__*/ React.createElement(Legends, {
            legends: actions,
            enabledWrapLines: props.enabledLegendsWrapLines,
            overflowText: props.legendsOverflowText,
            ...props.legendProps,
            onChange: onLegendSelectionChange
        });
    };
    const points = props.data;
    const { keys, xAxisLabels, datasetForBars } = _createDataSetOfGVBC(points);
    _keys = keys;
    _xAxisLabels = xAxisLabels;
    _datasetForBars = datasetForBars;
    const _xAxisType = getTypeOfAxis(points[0].name, true);
    const legends = _getLegendData(points);
    _adjustProps();
    // eslint-disable-next-line @typescript-eslint/no-explicit-anyAdd commentMore actions
    function _getMinMaxOfYAxis(datasetForBars, yAxisType, useSecondaryYScale) {
        const values = [];
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        datasetForBars.forEach((data)=>{
            data.groupSeries.forEach((point)=>{
                if (!useSecondaryYScale === !point.useSecondaryYScale) {
                    values.push(point.data);
                }
            });
        });
        return {
            startValue: d3Min(values),
            endValue: d3Max(values)
        };
    }
    // The maxOfYVal prop is only required for the primary y-axis, so yMax should be calculated
    // using only the data points associated with the primary y-axis.
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const yMax = _getMinMaxOfYAxis(_datasetForBars).endValue;
    _yMax = Math.max(yMax, props.yMaxValue || 0);
    var _props_culture;
    const calloutProps = {
        clickPosition,
        isPopoverOpen,
        color,
        legend: calloutLegend,
        XValue: xCalloutValue,
        YValue: yCalloutValue ? yCalloutValue : dataForHoverCard,
        YValueHover,
        hoverXValue,
        culture: (_props_culture = props.culture) !== null && _props_culture !== void 0 ? _props_culture : 'en-us',
        isCartesian: true,
        ...props.calloutProps,
        ...getAccessibleDataObject(callOutAccessibilityData, 'text', false)
    };
    const tickParams = {
        tickValues: props.tickValues,
        tickFormat: props.tickFormat
    };
    const _getGraphData = (xScale, yScalePrimary, containerHeight, containerWidth, xElement, yAxisElement, yScaleSecondary)=>{
        const xScale0 = _createX0Scale(containerWidth);
        // Setting the bar width here is safe because there are no dependencies earlier in the code
        // that rely on the width of bars in vertical bar charts with string x-axis.
        _barWidth = getBarWidth(props.barWidth, props.maxBarWidth, xScale0.bandwidth() / (_keys.length + (_keys.length - 1) * BAR_GAP_RATE));
        _groupWidth = (_keys.length + (_keys.length - 1) * BAR_GAP_RATE) * _barWidth;
        const xScale1 = _createX1Scale();
        const allGroupsBars = [];
        _datasetForBars.forEach((singleSet)=>{
            allGroupsBars.push(_buildGraph(singleSet, xScale0, xScale1, yScalePrimary, yScaleSecondary, containerHeight, xElement));
        });
        _groupedVerticalBarGraph = allGroupsBars;
    };
    const _getMargins = (margins)=>{
        _margins = margins;
    };
    const _getOpacity = (legendTitle)=>{
        const opacity = _legendHighlighted(legendTitle) || _noLegendHighlighted() ? '' : '0.1';
        return opacity;
    };
    function updatePosition(newX, newY) {
        const threshold = 1; // Set a threshold for movement
        const { x, y } = clickPosition;
        // Calculate the distance moved
        const distance = Math.sqrt(Math.pow(newX - x, 2) + Math.pow(newY - y, 2));
        // Update the position only if the distance moved is greater than the threshold
        if (distance > threshold) {
            setClickPosition({
                x: newX,
                y: newY
            });
            setPopoverOpen(true);
        }
    }
    const onBarHover = (pointData, // eslint-disable-next-line @typescript-eslint/no-explicit-any
    groupData, mouseEvent)=>{
        mouseEvent.persist();
        if (_calloutAnchorPoint !== pointData) {
            _calloutAnchorPoint = pointData;
            updatePosition(mouseEvent.clientX, mouseEvent.clientY);
            setPopoverOpen(_noLegendHighlighted() || _legendHighlighted(pointData.legend));
            setCalloutLegend(pointData.legend);
            setDataForHoverCard(pointData.data);
            setColor(pointData.color);
            setXCalloutValue(pointData.xAxisCalloutData);
            setYCalloutValue(pointData.yAxisCalloutData);
            setCallOutAccessibilityData(props.isCalloutForStack ? groupData.stackCallOutAccessibilityData : pointData.callOutAccessibilityData);
            setYValueHover(groupData.groupSeries);
            setHoverXValue(pointData.xAxisCalloutData);
        }
    };
    const _onBarLeave = ()=>{
    /**/ };
    const _handleChartMouseLeave = ()=>{
        _calloutAnchorPoint = null;
        setPopoverOpen(false);
    };
    const onBarFocus = (pointData, // eslint-disable-next-line @typescript-eslint/no-explicit-any
    groupData, refArrayIndexNumber)=>{
        _refArray.forEach((obj, index)=>{
            if (obj.index === pointData.legend && refArrayIndexNumber === index) {
                setPopoverOpen(_noLegendHighlighted() || _legendHighlighted(pointData.legend));
                setCalloutLegend(pointData.legend);
                setDataForHoverCard(pointData.data);
                setColor(pointData.color);
                setXCalloutValue(pointData.xAxisCalloutData);
                setYCalloutValue(pointData.yAxisCalloutData);
                setCallOutAccessibilityData(props.isCalloutForStack ? groupData.stackCallOutAccessibilityData : pointData.callOutAccessibilityData);
                setYValueHover(groupData.groupSeries);
                setHoverXValue(pointData.xAxisCalloutData);
            }
        });
    };
    const _refCallback = (element, legendTitle, refIndexNumber)=>{
        _refArray[refIndexNumber] = {
            index: legendTitle,
            refElement: element
        };
    };
    const _buildGraph = (// eslint-disable-next-line @typescript-eslint/no-explicit-any
    singleSet, // eslint-disable-next-line @typescript-eslint/no-explicit-any
    xScale0, // eslint-disable-next-line @typescript-eslint/no-explicit-any
    xScale1, yScalePrimary, yScaleSecondary, containerHeight, xElement)=>{
        const singleGroup = [];
        const barLabelsForGroup = [];
        const tempDataSet = Object.keys(datasetForBars[0]).splice(0, keys.length);
        tempDataSet.forEach((datasetKey, index)=>{
            const refIndexNumber = singleSet.indexNum * tempDataSet.length + index;
            const pointData = singleSet[datasetKey];
            const yBarScale = pointData.useSecondaryYScale && yScaleSecondary ? yScaleSecondary : yScalePrimary;
            // To align the centers of the generated bandwidth and the calculated one when they differ,
            // use the following addend.
            const xPoint = xScale1(datasetKey) + (xScale1.bandwidth() - _barWidth) / 2;
            const startColor = pointData.color ? pointData.color : getNextColor(index, 0);
            const yBaseline = yBarScale(Y_ORIGIN);
            let yPositiveStart = yBaseline;
            let yNegativeStart = yBaseline;
            let yPoint = Y_ORIGIN;
            const barGap = VERTICAL_BAR_GAP / 2 * (index > 0 ? 2 : 0);
            const height = Math.max(yBarScale(Y_ORIGIN) - yBarScale(Math.abs(pointData.data)), MIN_BAR_HEIGHT);
            if (pointData.data >= Y_ORIGIN) {
                yPositiveStart -= height + barGap;
                yPoint = yPositiveStart;
            } else {
                yPoint = yNegativeStart + barGap;
                yNegativeStart = yPoint + height;
            }
            // Not rendering data with 0.
            pointData.data && singleGroup.push(/*#__PURE__*/ React.createElement(React.Fragment, {
                key: `${singleSet.indexNum}-${index}`
            }, /*#__PURE__*/ React.createElement("rect", {
                className: classes.opacityChangeOnHover,
                height: height,
                width: _barWidth,
                x: xPoint,
                y: yPoint,
                "data-is-focusable": !props.hideTooltip && (_legendHighlighted(pointData.legend) || _noLegendHighlighted()),
                opacity: _getOpacity(pointData.legend),
                ref: (e)=>{
                    _refCallback(e, pointData.legend, refIndexNumber);
                },
                fill: startColor,
                rx: 0,
                onMouseOver: onBarHover.bind(null, pointData, singleSet),
                onMouseMove: onBarHover.bind(null, pointData, singleSet),
                onMouseOut: _onBarLeave,
                onFocus: onBarFocus.bind(null, pointData, singleSet, refIndexNumber),
                onBlur: _onBarLeave,
                onClick: pointData.onClick,
                "aria-label": getAriaLabel(pointData, singleSet.xAxisPoint),
                tabIndex: _legendHighlighted(pointData.legend) || _noLegendHighlighted() ? 0 : undefined,
                role: "img"
            })));
            if (pointData.data && !props.hideLabels && _barWidth >= 16 && (_legendHighlighted(pointData.legend) || _noLegendHighlighted())) {
                barLabelsForGroup.push(/*#__PURE__*/ React.createElement("text", {
                    key: `${singleSet.indexNum}-${index}`,
                    x: xPoint + _barWidth / 2,
                    y: pointData.data >= Y_ORIGIN ? yPositiveStart - 6 : yNegativeStart + 12,
                    textAnchor: "middle",
                    className: classes.barLabel,
                    "aria-hidden": true
                }, typeof props.yAxisTickFormat === 'function' ? props.yAxisTickFormat(pointData.data) : formatScientificLimitWidth(pointData.data)));
            }
        });
        // Used to display tooltip at x axis labels.
        if (!props.wrapXAxisLables && props.showXAxisLablesTooltip) {
            const xAxisElement = d3Select(xElement).call(xScale0);
            try {
                document.getElementById(_tooltipId) && document.getElementById(_tooltipId).remove();
            // eslint-disable-next-line no-empty
            } catch (e) {}
            const tooltipProps = {
                tooltipCls: classes.tooltip,
                id: _tooltipId,
                axis: xAxisElement
            };
            xAxisElement && tooltipOfAxislabels(tooltipProps);
        }
        return /*#__PURE__*/ React.createElement("g", {
            key: singleSet.indexNum,
            transform: `translate(${xScale0(singleSet.xAxisPoint) + (xScale0.bandwidth() - _groupWidth) / 2}, 0)`
        }, singleGroup, barLabelsForGroup);
    };
    // For grouped vertical bar chart, First need to define total scale (from start to end)
    // From that need to define scale for single group of bars - done by createX1Scale
    const _createX0Scale = (containerWidth)=>{
        const x0Axis = d3ScaleBand().domain(xAxisLabels).range(_useRtl ? [
            containerWidth - _margins.right - _domainMargin,
            _margins.left + _domainMargin
        ] : [
            _margins.left + _domainMargin,
            containerWidth - _margins.right - _domainMargin
        ]).paddingInner(_xAxisInnerPadding).paddingOuter(_xAxisOuterPadding);
        return x0Axis;
    };
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const _createX1Scale = ()=>{
        return d3ScaleBand().domain(keys)// When there is only one group, xScale0 adds padding around it,
        // causing the bandwidth to become smaller than the actual group width.
        // So to render bars in the group correctly, use groupWidth instead of the generated scale bandwidth.
        .range(_useRtl ? [
            _groupWidth,
            0
        ] : [
            0,
            _groupWidth
        ]).paddingInner(X1_INNER_PADDING);
    };
    const _onLegendHover = (legendTitle)=>{
        setActiveLegend(legendTitle);
    };
    const _onLegendLeave = ()=>{
        setActiveLegend('');
    };
    const _getAxisData = React.useCallback((yAxisData)=>{
        if (yAxisData && yAxisData.yAxisDomainValues.length) {
            const { yAxisDomainValues: domainValue } = yAxisData;
            _yMax = Math.max(domainValue[domainValue.length - 1], props.yMaxValue || 0);
        }
    }, [
        props.yMaxValue
    ]);
    /**
   * This function checks if the given legend is highlighted or not.
   * A legend can be highlighted in 2 ways:
   * 1. selection: if the user clicks on it
   * 2. hovering: if there is no selected legend and the user hovers over it
   */ const _legendHighlighted = (legendTitle)=>{
        return _getHighlightedLegend().includes(legendTitle);
    };
    /**
   * This function checks if none of the legends is selected or hovered.
   */ const _noLegendHighlighted = ()=>{
        return _getHighlightedLegend().length === 0;
    };
    const _getHighlightedLegend = ()=>{
        return selectedLegends.length > 0 ? selectedLegends : activeLegend ? [
            activeLegend
        ] : [];
    };
    const getAriaLabel = (point, xAxisPoint)=>{
        var _point_callOutAccessibilityData;
        const xValue = point.xAxisCalloutData || xAxisPoint;
        const legend = point.legend;
        const yValue = point.yAxisCalloutData || point.data;
        return ((_point_callOutAccessibilityData = point.callOutAccessibilityData) === null || _point_callOutAccessibilityData === void 0 ? void 0 : _point_callOutAccessibilityData.ariaLabel) || `${xValue}. ${legend}, ${yValue}.`;
    };
    const _getDomainMargins = (containerWidth)=>{
        /** Total width available to render the bars */ const totalWidth = containerWidth - (_margins.left + MIN_DOMAIN_MARGIN) - (_margins.right + MIN_DOMAIN_MARGIN);
        /** Rate at which the space between the groups changes wrt the group width */ const groupGapRate = _xAxisInnerPadding / (1 - _xAxisInnerPadding);
        if (_xAxisType === XAxisTypes.StringAxis) {
            if (isScalePaddingDefined(props.xAxisOuterPadding)) {
                // Setting the domain margin for string x-axis to 0 because the xAxisOuterPadding prop is now available
                // to adjust the space before the first group and after the last group.
                _domainMargin = 0;
            } else if (props.barWidth !== 'auto') {
                // Update the bar width so that when CartesianChart rerenders,
                // the following calculations don't use the previous bar width.
                _barWidth = getBarWidth(props.barWidth, props.maxBarWidth);
                const groupWidth = (keys.length + (keys.length - 1) * BAR_GAP_RATE) * _barWidth;
                /** Total width required to render the groups. Directly proportional to group width */ const reqWidth = (xAxisLabels.length + (xAxisLabels.length - 1) * groupGapRate) * groupWidth;
                if (totalWidth >= reqWidth) {
                    // Center align the chart by setting equal left and right margins for domain
                    _domainMargin = MIN_DOMAIN_MARGIN + (totalWidth - reqWidth) / 2;
                }
            } else if (props.mode === 'plotly' && xAxisLabels.length > 1) {
                // Calculate the remaining width after rendering groups at their maximum allowable width
                const groupBandwidth = totalWidth / (xAxisLabels.length + (xAxisLabels.length - 1) * groupGapRate);
                const barBandwidth = groupBandwidth / (keys.length + (keys.length - 1) * BAR_GAP_RATE);
                const barWidth = getBarWidth(props.barWidth, props.maxBarWidth, barBandwidth);
                const groupWidth = (keys.length + (keys.length - 1) * BAR_GAP_RATE) * barWidth;
                let reqWidth = (xAxisLabels.length + (xAxisLabels.length - 1) * groupGapRate) * groupWidth;
                const margin1 = (totalWidth - reqWidth) / 2;
                // Calculate the remaining width after accounting for the space required to render x-axis labels
                const step = calculateLongestLabelWidth(xAxisLabels) + 20;
                reqWidth = (xAxisLabels.length - _xAxisInnerPadding) * step;
                const margin2 = (totalWidth - reqWidth) / 2;
                _domainMargin = MIN_DOMAIN_MARGIN + Math.max(0, Math.min(margin1, margin2));
            }
        }
        return {
            ..._margins,
            left: _margins.left + _domainMargin,
            right: _margins.right + _domainMargin
        };
    };
    const _isChartEmpty = ()=>{
        return !(props.data && props.data.length > 0 && props.data.filter((item)=>item.series.length).length > 0);
    };
    const _getChartTitle = ()=>{
        return (props.chartTitle ? `${props.chartTitle}. ` : '') + `Vertical bar chart with ${_xAxisLabels.length} groups of ${_keys.length} bars each. `;
    };
    return !_isChartEmpty() ? /*#__PURE__*/ React.createElement(CartesianChart, {
        ...props,
        chartTitle: _getChartTitle(),
        points: _datasetForBars,
        chartType: ChartTypes.GroupedVerticalBarChart,
        getMinMaxOfYAxis: _getMinMaxOfYAxis,
        calloutProps: calloutProps,
        legendBars: legends,
        xAxisType: _xAxisType,
        datasetForXAxisDomain: _xAxisLabels,
        tickParams: tickParams,
        tickPadding: props.tickPadding || 5,
        maxOfYVal: _yMax,
        getmargins: _getMargins,
        getGraphData: _getGraphData,
        getAxisData: _getAxisData,
        onChartMouseLeave: _handleChartMouseLeave,
        getDomainMargins: _getDomainMargins,
        ..._xAxisType === XAxisTypes.StringAxis && {
            xAxisInnerPadding: _xAxisInnerPadding,
            xAxisOuterPadding: _xAxisOuterPadding
        },
        barwidth: _barWidth,
        componentRef: cartesianChartRef,
        /* eslint-disable react/jsx-no-bind */ children: ()=>{
            return /*#__PURE__*/ React.createElement("g", null, _groupedVerticalBarGraph);
        }
    }) : /*#__PURE__*/ React.createElement("div", {
        id: _emptyChartId,
        role: 'alert',
        style: {
            opacity: '0'
        },
        "aria-label": 'Graph has no data to display'
    });
});
GroupedVerticalBarChart.displayName = 'GroupedVerticalBarChart';
