import * as React from 'react';
import { useScatterChartStyles_unstable } from './useScatterChartStyles.styles';
import { select as d3Select } from 'd3-selection';
import { Legends } from '../Legends/index';
import { max as d3Max, min as d3Min } from 'd3-array';
import { useId } from '@fluentui/react-utilities';
import { areArraysEqual, find } from '../../utilities/index';
import { CartesianChart } from '../../index';
import { tokens } from '@fluentui/react-theme';
import { calloutData, ChartTypes, XAxisTypes, tooltipOfAxislabels, getTypeOfAxis, getNextColor, getColorFromToken, formatDate } from '../../utilities/index';
// Create a ScatterChart variant which uses these default styles and this styled subcomponent.
/**
 * ScatterChart component
 * {@docCategory ScatterChart}
 */ export const ScatterChart = /*#__PURE__*/ React.forwardRef((props, forwardedRef)=>{
    var _props_legendProps, _props_legendProps1;
    const _circleId = useId('circle');
    const _seriesId = useId('seriesID');
    const _verticalLine = useId('verticalLine');
    const _tooltipId = useId('ScatterChartTooltipId_');
    const _firstRenderOptimization = true;
    const _emptyChartId = useId('_ScatterChart_empty');
    let _points = _injectIndexPropertyInScatterChartData(props.data.lineChartData);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    let _calloutPoints = calloutData(_points) || [];
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    let _xAxisScale = '';
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    let _yAxisScale = '';
    let _uniqueCallOutID = '';
    let _refArray = [];
    let margins;
    let renderSeries;
    let _xAxisLabels = [];
    let xAxisCalloutAccessibilityData = {};
    let _xBandwidth = 0;
    const cartesianChartRef = React.useRef(null);
    const [hoverXValue, setHoverXValue] = React.useState('');
    const [activeLegend, setActiveLegend] = React.useState('');
    const [YValueHover, setYValueHover] = React.useState([]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const [selectedLegendPoints, setSelectedLegendPoints] = React.useState([]);
    const [isSelectedLegend, setIsSelectedLegend] = React.useState(false);
    const [activePoint, setActivePoint] = React.useState('');
    const [stackCalloutProps, setStackCalloutProps] = React.useState();
    const [clickPosition, setClickPosition] = React.useState({
        x: 0,
        y: 0
    });
    const [isPopoverOpen, setPopoverOpen] = React.useState(false);
    const [selectedLegends, setSelectedLegends] = React.useState(((_props_legendProps = props.legendProps) === null || _props_legendProps === void 0 ? void 0 : _props_legendProps.selectedLegends) || []);
    const prevSelectedLegendsRef = React.useRef(undefined);
    React.useEffect(()=>{
        var _props_legendProps, _props_legendProps1;
        if (prevSelectedLegendsRef.current && !areArraysEqual(prevSelectedLegendsRef.current, (_props_legendProps = props.legendProps) === null || _props_legendProps === void 0 ? void 0 : _props_legendProps.selectedLegends)) {
            var _props_legendProps2;
            setSelectedLegends(((_props_legendProps2 = props.legendProps) === null || _props_legendProps2 === void 0 ? void 0 : _props_legendProps2.selectedLegends) || []);
        }
        prevSelectedLegendsRef.current = (_props_legendProps1 = props.legendProps) === null || _props_legendProps1 === void 0 ? void 0 : _props_legendProps1.selectedLegends;
    }, [
        (_props_legendProps1 = props.legendProps) === null || _props_legendProps1 === void 0 ? void 0 : _props_legendProps1.selectedLegends
    ]);
    React.useImperativeHandle(props.componentRef, ()=>{
        var _cartesianChartRef_current;
        var _cartesianChartRef_current_chartContainer;
        return {
            chartContainer: (_cartesianChartRef_current_chartContainer = (_cartesianChartRef_current = cartesianChartRef.current) === null || _cartesianChartRef_current === void 0 ? void 0 : _cartesianChartRef_current.chartContainer) !== null && _cartesianChartRef_current_chartContainer !== void 0 ? _cartesianChartRef_current_chartContainer : null
        };
    }, []);
    const _xAxisType = props.data.lineChartData && props.data.lineChartData.length > 0 && props.data.lineChartData[0].data && props.data.lineChartData[0].data.length > 0 ? getTypeOfAxis(props.data.lineChartData[0].data[0].x, true) : XAxisTypes.StringAxis;
    const pointsRef = React.useRef([]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const calloutPointsRef = React.useRef([]);
    React.useEffect(()=>{
        /** note that height and width are not used to resize or set as dimesions of the chart,
     * fitParentContainer is responisble for setting the height and width or resizing of the svg/chart
     */ if (_points !== _injectIndexPropertyInScatterChartData(props.data.lineChartData) || props.data !== _points) {
            pointsRef.current = _injectIndexPropertyInScatterChartData(props.data.lineChartData);
            calloutPointsRef.current = calloutData(pointsRef.current);
        }
    }, [
        props.height,
        props.width,
        props.data,
        _points
    ]);
    function _injectIndexPropertyInScatterChartData(scatterChartData) {
        return scatterChartData ? scatterChartData.map((item, index)=>{
            let color;
            if (typeof item.color === 'undefined') {
                color = getNextColor(index, 0);
            } else {
                color = getColorFromToken(item.color);
            }
            return {
                ...item,
                index: -1,
                color
            };
        }) : [];
    }
    function updatePosition(newX, newY) {
        const threshold = 1; // Set a threshold for movement
        const { x, y } = clickPosition;
        // Calculate the distance moved
        const distance = Math.sqrt(Math.pow(newX - x, 2) + Math.pow(newY - y, 2));
        // Update the position only if the distance moved is greater than the threshold
        if (distance > threshold) {
            setClickPosition({
                x: newX,
                y: newY
            });
            setPopoverOpen(true);
        }
    }
    function _getMargins(_margins) {
        margins = _margins;
    }
    function _initializeScatterChartData(xScale, yScale, containerHeight, containerWidth, xElement) {
        _xAxisScale = xScale;
        _yAxisScale = yScale;
        renderSeries = _createPlot(xElement, containerHeight);
    }
    function _onHoverCardHide() {
        setSelectedLegendPoints([]);
        setIsSelectedLegend(false);
    }
    function _createLegends(data) {
        const { legendProps } = props;
        const isLegendMultiSelectEnabled = !!(legendProps && !!legendProps.canSelectMultipleLegends);
        const legendDataItems = data.map((point)=>{
            const color = point.color;
            // mapping data to the format Legends component needs
            const legend = {
                title: point.legend,
                color,
                onMouseOutAction: ()=>{
                    setActiveLegend('');
                },
                hoverAction: ()=>{
                    _handleChartMouseLeave();
                    setActiveLegend(point.legend);
                },
                ...point.legendShape && {
                    shape: point.legendShape
                }
            };
            return legend;
        });
        return /*#__PURE__*/ React.createElement(Legends, {
            legends: [
                ...legendDataItems
            ],
            enabledWrapLines: props.enabledLegendsWrapLines,
            overflowText: props.legendsOverflowText,
            ...isLegendMultiSelectEnabled && {
                onLegendHoverCardLeave: _onHoverCardHide
            },
            ...props.legendProps,
            selectedLegends: selectedLegends,
            onChange: _onLegendSelectionChange
        });
    }
    function _onLegendSelectionChange(legendsSelected, event, currentLegend) {
        var _props_legendProps, _props_legendProps1;
        if ((_props_legendProps = props.legendProps) === null || _props_legendProps === void 0 ? void 0 : _props_legendProps.canSelectMultipleLegends) {
            setSelectedLegends(legendsSelected);
        } else {
            setSelectedLegends(legendsSelected.slice(-1));
        }
        if ((_props_legendProps1 = props.legendProps) === null || _props_legendProps1 === void 0 ? void 0 : _props_legendProps1.onChange) {
            props.legendProps.onChange(legendsSelected, event, currentLegend);
        }
    }
    function _getPointFill(seriesColor, pointId, pointIndex, isLastPoint) {
        if (activePoint === pointId) {
            return tokens.colorNeutralBackground1;
        } else {
            return seriesColor;
        }
    }
    function _createPlot(xElement, containerHeight) {
        const series = [];
        if (isSelectedLegend) {
            _points = selectedLegendPoints;
        } else {
            _points = _injectIndexPropertyInScatterChartData(props.data.lineChartData);
        }
        const yMax = d3Max(points, (point)=>{
            return d3Max(point.data, (item)=>item.y);
        });
        const yMin = d3Min(points, (point)=>{
            return d3Min(point.data, (item)=>item.y);
        });
        const yPadding = (yMax - yMin) * 0.1;
        const yPaddingRange = Math.abs(_yAxisScale(yMin + yPadding) - _yAxisScale(yMin));
        let maxMarkerRange = 40;
        let xPaddingRange = 0;
        if (_xAxisType === XAxisTypes.StringAxis) {
            _xBandwidth = _xAxisScale.bandwidth() / 2;
            xPaddingRange = _xBandwidth;
        } else if (_xAxisType === XAxisTypes.DateAxis) {
            const xMin = d3Min(points, (point)=>{
                return d3Min(point.data, (item)=>item.x);
            });
            const xMax = d3Max(points, (point)=>{
                return d3Max(point.data, (item)=>{
                    return item.x;
                });
            });
            const xPadding = (xMax.getTime() - xMin.getTime()) * 0.1;
            xPaddingRange = Math.abs(_xAxisScale(new Date(xMin.getTime() + xPadding)) - _xAxisScale(xMin));
        } else {
            const xMin = d3Min(points, (point)=>{
                return d3Min(point.data, (item)=>item.x);
            });
            const xMax = d3Max(points, (point)=>{
                return d3Max(point.data, (item)=>{
                    return item.x;
                });
            });
            const xPadding = (xMax - xMin) * 0.1;
            xPaddingRange = Math.abs(_xAxisScale(xMin + xPadding) - _xAxisScale(xMin));
        }
        maxMarkerRange = Math.min(maxMarkerRange, Math.min(xPaddingRange, yPaddingRange));
        const maxMarkerSize = d3Max(_points, (point)=>{
            return d3Max(point.data, (item)=>{
                return item.markerSize;
            });
        });
        for(let i = _points.length - 1; i >= 0; i--){
            const pointsForSeries = [];
            const legendVal = _points[i].legend;
            const seriesColor = _points[i].color;
            const verticaLineHeight = containerHeight - margins.bottom + 6;
            for(let j = 0; j < _points[i].data.length; j++){
                const seriesId = `${_seriesId}_${i}_${j}`;
                const circleId = `${_circleId}_${i}_${j}`;
                const { x, y, xAxisCalloutData, xAxisCalloutAccessibilityData } = _points[i].data[j];
                let circleRadius = 3.5;
                const pointMarkerSize = _points[i].data[j].markerSize;
                if (pointMarkerSize !== undefined) {
                    circleRadius = Math.min(pointMarkerSize * maxMarkerRange / maxMarkerSize, pointMarkerSize);
                }
                const isLegendSelected = _legendHighlighted(legendVal) || _noLegendHighlighted() || isSelectedLegend;
                const currentPointHidden = _points[i].hideNonActiveDots && activePoint !== circleId;
                pointsForSeries.push(/*#__PURE__*/ React.createElement("circle", {
                    id: circleId,
                    key: circleId,
                    r: circleRadius,
                    cx: _xAxisScale(x) + _xBandwidth,
                    cy: _yAxisScale(y),
                    "data-is-focusable": isLegendSelected,
                    onMouseOver: (event)=>_handleHover(x, y, verticaLineHeight, xAxisCalloutData, circleId, xAxisCalloutAccessibilityData, event),
                    onMouseMove: (event)=>_handleHover(x, y, verticaLineHeight, xAxisCalloutData, circleId, xAxisCalloutAccessibilityData, event),
                    onMouseOut: _handleMouseOut,
                    onFocus: ()=>_handleFocus(seriesId, x, xAxisCalloutData, circleId, xAxisCalloutAccessibilityData),
                    onBlur: _handleMouseOut,
                    ..._getClickHandler(_points[i].data[j].onDataPointClick),
                    opacity: isLegendSelected && !currentPointHidden ? 1 : 0.1,
                    fill: _getPointFill(seriesColor, circleId, j, false),
                    stroke: seriesColor,
                    role: "img",
                    "aria-label": _getAriaLabel(i, j),
                    tabIndex: isLegendSelected ? 0 : undefined
                }));
            }
            series.push(/*#__PURE__*/ React.createElement("g", {
                key: `series_${i}`,
                role: "region",
                "aria-label": `${legendVal}, series ${i + 1} of ${_points.length} with ${_points[i].data.length} data points.`
            }, pointsForSeries));
        }
        const classes = useScatterChartStyles_unstable(props);
        // Removing un wanted tooltip div from DOM, when prop not provided.
        if (!props.showXAxisLablesTooltip) {
            try {
                document.getElementById(_tooltipId) && document.getElementById(_tooltipId).remove();
            // eslint-disable-next-line no-empty
            } catch (e) {}
        }
        // Used to display tooltip at x axis labels.
        if (!props.wrapXAxisLables && props.showXAxisLablesTooltip) {
            const xAxisElement = d3Select(xElement).call(_xAxisScale);
            try {
                document.getElementById(_tooltipId) && document.getElementById(_tooltipId).remove();
            // eslint-disable-next-line no-empty
            } catch (e) {}
            const tooltipProps = {
                tooltipCls: classes.tooltip,
                id: _tooltipId,
                axis: xAxisElement
            };
            xAxisElement && tooltipOfAxislabels(tooltipProps);
        }
        return series;
    }
    function _handleFocus(seriesId, x, xAxisCalloutData, circleId, xAxisCalloutAccessibilityData) {
        _uniqueCallOutID = circleId;
        const formattedData = x instanceof Date ? formatDate(x, props.useUTC) : x;
        const xVal = x instanceof Date ? x.getTime() : x;
        const found = find(_calloutPoints, (element)=>element.x === xVal);
        // if no points need to be called out then don't show vertical line and callout card
        if (found) {
            d3Select(`#${_verticalLine}`).attr('transform', ()=>`translate(${_xAxisScale(x) + _xBandwidth}, 0)`).attr('visibility', 'visibility');
            _refArray.forEach((obj)=>{
                if (obj.index === seriesId) {
                    setPopoverOpen(true);
                    xAxisCalloutData ? setHoverXValue(xAxisCalloutData) : setHoverXValue('' + formattedData);
                    setYValueHover(found.values);
                    setStackCalloutProps(found);
                    setActivePoint(circleId);
                }
            });
        } else {
            setActivePoint(circleId);
        }
    }
    function _handleHover(x, y, lineHeight, xAxisCalloutData, circleId, xAxisCalloutAccessibilityData, mouseEvent) {
        mouseEvent === null || mouseEvent === void 0 ? void 0 : mouseEvent.persist();
        const formattedData = x instanceof Date ? formatDate(x, props.useUTC) : x;
        const xVal = x instanceof Date ? x.getTime() : x;
        const found = find(_calloutPoints, (element)=>element.x === xVal);
        // if no points need to be called out then don't show vertical line and callout card
        if (found) {
            d3Select(`#${_verticalLine}`).attr('transform', ()=>`translate(${_xAxisScale(x) + _xBandwidth}, ${_yAxisScale(y)})`).attr('visibility', 'visibility').attr('y2', `${lineHeight - _yAxisScale(y)}`);
            if (_uniqueCallOutID !== circleId) {
                _uniqueCallOutID = circleId;
                updatePosition(mouseEvent.clientX, mouseEvent.clientY);
                xAxisCalloutData ? setHoverXValue(xAxisCalloutData) : setHoverXValue('' + formattedData);
                setYValueHover(found.values);
                setStackCalloutProps(found);
                setActivePoint(circleId);
            }
        } else {
            setActivePoint(circleId);
        }
    }
    /**
   * Screen readers announce an element as clickable if the onClick attribute is set.
   * This function sets the attribute only when a click event handler is provided.*/ function _getClickHandler(func) {
        if (func) {
            return {
                onClick: func
            };
        }
        return {};
    }
    function _handleMouseOut() {
        d3Select(`#${_verticalLine}`).attr('visibility', 'hidden');
    }
    function _handleChartMouseLeave() {
        _uniqueCallOutID = null;
        setActivePoint('');
        if (isPopoverOpen) {
            setPopoverOpen(false);
        }
    }
    /**
   * This function checks if the given legend is highlighted or not.
   * A legend can be highlighted in 2 ways:
   * 1. selection: if the user clicks on it
   * 2. hovering: if there is no selected legend and the user hovers over it*/ function _legendHighlighted(legend) {
        return _getHighlightedLegend().includes(legend);
    }
    /**
   * This function checks if none of the legends is selected or hovered.*/ function _noLegendHighlighted() {
        return _getHighlightedLegend().length === 0;
    }
    function _getHighlightedLegend() {
        return selectedLegends.length > 0 ? selectedLegends : activeLegend ? [
            activeLegend
        ] : [];
    }
    function _getAriaLabel(seriesIndex, pointIndex) {
        var _point_callOutAccessibilityData;
        const series = _points[seriesIndex];
        const point = series.data[pointIndex];
        const formattedDate = point.x instanceof Date ? formatDate(point.x, props.useUTC) : point.x;
        const xValue = point.xAxisCalloutData || formattedDate;
        const legend = series.legend;
        const yValue = point.yAxisCalloutData || point.y;
        return ((_point_callOutAccessibilityData = point.callOutAccessibilityData) === null || _point_callOutAccessibilityData === void 0 ? void 0 : _point_callOutAccessibilityData.ariaLabel) || `${xValue}. ${legend}, ${yValue}.`;
    }
    function _isChartEmpty() {
        return !(props.data && props.data.lineChartData && props.data.lineChartData.length > 0 && props.data.lineChartData.filter((item)=>item.data.length).length > 0);
    }
    const { legendProps, tickValues, tickFormat } = props;
    _points = _injectIndexPropertyInScatterChartData(props.data.lineChartData);
    let points = _points;
    if (legendProps && !!legendProps.canSelectMultipleLegends) {
        points = selectedLegendPoints.length >= 1 ? selectedLegendPoints : _points;
        _calloutPoints = calloutData(points);
    }
    let legendBars = null;
    // reduce computation cost by only creating legendBars
    // if when hideLegend is false.
    // NOTE: they are rendered only when hideLegend is false in CartesianChart.
    if (!props.hideLegend) {
        legendBars = _createLegends(_points); // ToDo: Memoize legends to improve performance.
    }
    var _props_culture;
    const calloutProps = {
        YValueHover,
        hoverXValue,
        descriptionMessage: props.getCalloutDescriptionMessage && stackCalloutProps ? props.getCalloutDescriptionMessage(stackCalloutProps) : undefined,
        'data-is-focusable': true,
        xAxisCalloutAccessibilityData,
        ...props.calloutProps,
        clickPosition,
        isPopoverOpen,
        isCalloutForStack: true,
        culture: (_props_culture = props.culture) !== null && _props_culture !== void 0 ? _props_culture : 'en-us',
        isCartesian: true
    };
    const tickParams = {
        tickValues,
        tickFormat
    };
    const xAxisLabels = _points.map((point)=>point.data.map((dp)=>dp.x)).flat();
    _xAxisLabels = [
        ...new Set(xAxisLabels)
    ];
    return !_isChartEmpty() ? /*#__PURE__*/ React.createElement(CartesianChart, {
        ...props,
        chartTitle: props.data.chartTitle,
        points: points,
        chartType: ChartTypes.ScatterChart,
        calloutProps: calloutProps,
        tickParams: tickParams,
        legendBars: legendBars,
        getmargins: _getMargins,
        getGraphData: _initializeScatterChartData,
        xAxisType: _xAxisType,
        onChartMouseLeave: _handleChartMouseLeave,
        enableFirstRenderOptimization: _firstRenderOptimization,
        datasetForXAxisDomain: _xAxisLabels,
        componentRef: cartesianChartRef,
        /* eslint-disable react/jsx-no-bind */ // eslint-disable-next-line react/no-children-prop
        children: (props)=>{
            _xAxisScale = props.xScale;
            _yAxisScale = props.yScalePrimary;
            return /*#__PURE__*/ React.createElement(React.Fragment, null, /*#__PURE__*/ React.createElement("g", null, /*#__PURE__*/ React.createElement("line", {
                x1: 0,
                y1: 0,
                x2: 0,
                y2: props.containerHeight,
                stroke: '#323130',
                id: _verticalLine,
                visibility: 'hidden',
                strokeDasharray: '5,5'
            }), /*#__PURE__*/ React.createElement("g", null, renderSeries)));
        }
    }) : /*#__PURE__*/ React.createElement("div", {
        id: _emptyChartId,
        role: 'alert',
        style: {
            opacity: '0'
        },
        "aria-label": 'Graph has no data to display'
    });
});
ScatterChart.displayName = 'ScatterChart';
